use std::{cell::RefCell, rc::Rc};

use gtk4::{
    glib,
    prelude::{ApplicationExt, GtkWindowExt, WidgetExt},
    Application, ApplicationWindow, Builder, Label,
};
use log::trace;

use crate::protocol::BackToFront;

pub(crate) fn get_app(
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) -> Application {
    let app = Application::builder()
        .application_id("de.teilgedanken.sieverman")
        .build();

    app.connect_activate(move |app| build_ui(app, from_backend_rx.clone()));

    app
}

fn build_ui(
    app: &gtk4::Application,
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) {
    let glade_src = include_str!("../../../gui/sieverman.ui");
    let builder = Builder::from_string(glade_src);
    let main_window: ApplicationWindow = builder.object("sieverman").expect("get main-window");
    main_window.set_height_request(400);
    main_window.set_application(Some(app));
    main_window.present();

    let future = {
        let mut data_event_receiver = from_backend_rx.take().expect("data_event_reciver");
        async move {
            while let Some(event) = data_event_receiver.recv().await {
                trace!("data event: {:?}", event);
                match event {
                    BackToFront::ServerConnected(message) => {
                        trace!("Received {}", message);
                        let status_label: Label = builder.object("serverstatus").unwrap();
                        status_label.set_text(&message)
                    }
                }
            }
        }
    };

    let c = glib::MainContext::default();
    c.spawn_local(future);
    main_window.present();
}
